@@ -148,6 +148,7 @@ class Experiment(BaseExperiment):
         display_summary_level=1,
         optimizer_data=None,
         auto_weight_logging=False,
+        auto_log_co2=True,
     ):
         """
         Creates a new experiment on the Comet.ml frontend.
@@ -170,6 +171,8 @@ class Experiment(BaseExperiment):
                 `"default"` which will detect your environment and deactivate
                 the output logging for IPython and Jupyter environment and
                 sets `"native"` in the other cases.
+            auto_log_co2: Default(True) - automatically tracks the CO2 emission of
+                this experiment if `codecarbon` package is installed in the environment
             parse_args: Default(True) - allows you to enable/disable automatic parsing of CLI arguments
             log_env_details: Default(True) - log various environment
                 information in order to identify where the script is running
@@ -224,6 +227,7 @@ class Experiment(BaseExperiment):
             log_env_cpu=log_env_cpu,
             optimizer_data=optimizer_data,
             auto_weight_logging=auto_weight_logging,
+            auto_log_co2=auto_log_co2,
         )
 
         self.ws_connection = None
@@ -737,6 +741,8 @@ class ExistingExperiment(Experiment):
                 `"default"` which will detect your environment and deactivate
                 the output logging for IPython and Jupyter environment and
                 sets `"native"` in the other cases.
+            auto_log_co2: Default(True) - automatically tracks the CO2 emission of
+                this experiment if `codecarbon` package is installed in the environment
             parse_args: Default(False) - allows you to enable/disable automatic parsing of CLI arguments
             log_env_details: Default(False) - log various environment
                 information in order to identify where the script is running
