@@ -204,7 +204,7 @@ class Config(object):
             return config_default
 
     def get_string(self, user_value, config_name, default=None, not_set_value=None):
-        # type: (Any, str, Optional[str]) -> str
+        # type: (Any, str, Optional[str], Any) -> str
         """
         Returns the correct config value based on the following priority list:
         * User_value if set and not None
@@ -226,7 +226,7 @@ class Config(object):
         return value
 
     def get_bool(self, user_value, config_name, default=None, not_set_value=None):
-        # type: (Any, str, Optional[bool]) -> bool
+        # type: (Any, str, Optional[bool], Any) -> bool
         """
         Returns the correct config value based on the following priority list:
         * User_value if set and not None
@@ -247,6 +247,28 @@ class Config(object):
 
         return value
 
+    def get_int(self, user_value, config_name, default=None, not_set_value=None):
+        # type: (Any, str, Optional[int], int) -> bool
+        """
+        Returns the correct config value based on the following priority list:
+        * User_value if set and not None
+        * The override value from the Backend
+        * The configured value
+        * The default value passed in argument if not None
+        * The configured value default
+
+        In addition make sure the returned value is an int
+        """
+
+        value = self.get_raw(
+            user_value=user_value,
+            config_name=config_name,
+            default=default,
+            not_set_value=not_set_value,
+        )
+
+        return value
+
     def get_subsections(self):
         """
         Return the subsection config names.
@@ -480,6 +502,9 @@ CONFIG_MAP = {
     "comet.auto_log.output_logger": {"type": str},
     "comet.auto_log.parameters": {"type": bool},
     "comet.auto_log.weights": {"type": bool},
+    "comet.auto_log.co2": {"type": bool},
+    # Internals
+    "comet.internal.reporting": {"type": bool, "default": True},
 }
 
 
