@@ -38,7 +38,7 @@ from ._typing import (
     UserText,
     ValidFilePath,
 )
-from .config import get_config
+from .config import Config, get_config
 from .connection import Reporting, get_http_session
 from .exceptions import FileIsTooBig
 from .logging_messages import (
@@ -79,12 +79,10 @@ def compress_git_patch(git_patch):
     return archive, zip_path
 
 
-def _send_file(url, files, params, experiment_id):
-    # type: (str, Any, Dict[str, str], str) -> Response
+def _send_file(url, files, params, experiment_id, config):
+    # type: (str, Any, Dict[str, str], str, Config) -> Response
     LOGGER.debug("Uploading files %r to %s with params %s", files, url, params)
 
-    config = get_config()
-
     with get_http_session() as session:
         r = session.post(
             url,
@@ -110,10 +108,11 @@ def send_file(
     experiment_id,
     project_id,
     file_path,
+    config,
     additional_params=None,
     metadata=None,
 ):
-    # type: (str, str, str, str, str, Optional[Dict[str, str]], Optional[Dict[Any, Any]]) -> Response
+    # type: (str, str, str, str, str, Config, Optional[Dict[str, str]], Optional[Dict[Any, Any]]) -> Response
     params = {"experimentId": experiment_id, "projectId": project_id, "apiKey": api_key}
 
     if additional_params is not None:
@@ -128,7 +127,11 @@ def send_file(
                 files["metadata"] = encoded_metadata
 
         return _send_file(
-            post_endpoint, params=params, files=files, experiment_id=experiment_id
+            post_endpoint,
+            params=params,
+            files=files,
+            experiment_id=experiment_id,
+            config=config,
         )
 
 
@@ -138,10 +141,11 @@ def send_file_like(
     experiment_id,
     project_id,
     file_like,
+    config,
     additional_params=None,
     metadata=None,
 ):
-    # type: (str, str, str, str, Any, Optional[Dict[str, str]], Optional[Dict[Any, Any]]) -> Response
+    # type: (str, str, str, str, Any, Config, Optional[Dict[str, str]], Optional[Dict[Any, Any]]) -> Response
     params = {"experimentId": experiment_id, "projectId": project_id, "apiKey": api_key}
 
     if additional_params is not None:
@@ -154,7 +158,11 @@ def send_file_like(
             files["metadata"] = encoded_metadata
 
     return _send_file(
-        post_endpoint, params=params, files=files, experiment_id=experiment_id
+        post_endpoint,
+        params=params,
+        files=files,
+        experiment_id=experiment_id,
+        config=config,
     )
 
 
@@ -168,6 +176,8 @@ def upload_file(
     metadata=None,
     clean=True,
 ):
+    config = get_config()
+
     try:
         response = send_file(
             upload_endpoint,
@@ -177,6 +187,7 @@ def upload_file(
             file_path,
             additional_params=additional_params,
             metadata=metadata,
+            config=config,
         )
 
         if clean is True:
@@ -199,6 +210,7 @@ def upload_file(
             project_id=project_id,
             api_key=api_key,
             err_msg=str(e),
+            config=config,
         )
 
 
@@ -211,6 +223,8 @@ def upload_file_like(
     additional_params=None,
     metadata=None,
 ):
+    config = get_config()
+
     try:
         response = send_file_like(
             upload_endpoint,
@@ -218,6 +232,7 @@ def upload_file_like(
             experiment_id,
             project_id,
             file_like,
+            config,
             additional_params=additional_params,
             metadata=metadata,
         )
@@ -235,6 +250,7 @@ def upload_file_like(
             project_id=project_id,
             api_key=api_key,
             err_msg=str(e),
+            config=config,
         )
 
 
