@@ -159,3 +159,15 @@ MLFLOW_OFFLINE_EXPERIMENT_FALLBACK = "No Comet API Key was found, creating an Of
 MLFLOW_RESUMED_RUN = "Resumed MLFlow run are tracked as new Comet.ml Experiments"
 
 ONLINE_EXPERIMENT_THROTTLED = "Experiment has been throttled, some data will be missing"
+
+TF_KERAS_FALLBACK_FAILED = (
+    "Keras is not available; attempted to fallback to tensorflow.keras but still failed"
+)
+
+CODECARBON_NOT_INSTALLED = "codecarbon package not installed; skipping co2 tracking"
+
+CODECARBON_START_FAILED = "Failed to set-up CO2 logger"
+
+CODECARBON_STOP_FAILED = "Failed to shutdown the CO2Tracker properly"
+
+CODECARBON_DIR_CREATION_FAILED = "Failed to create the temporary CO2 tracker directory %r; skipping CO2 tracking; error %r"
