@@ -23,6 +23,7 @@ from .connection import (
     sanitize_url,
     url_join,
 )
+from .exceptions import INVALID_OPTIMIZATION_ID, CometRestApiException
 
 LOGGER = logging.getLogger(__name__)
 
@@ -362,7 +363,7 @@ class Predictor(object):
                 response_data = self.api.get_optimizer_best(
                     self.experiment.id, self.optimizer_id, self.loss_name, maximum=False
                 )
-            except Exception as e:
+            except Exception as exc:
                 if count == self.MAX_TRIES:
                     LOGGER.error(
                         "Max retries exceeded while trying to fetch "
@@ -371,9 +372,14 @@ class Predictor(object):
                     )
                     return None
 
-                LOGGER.debug(
-                    "Error while getting the optimizer best %s", e, exc_info=True
-                )
+                sdk_error_code = None
+                if isinstance(exc, CometRestApiException) and exc.safe_json_response:
+                    sdk_error_code = exc.safe_json_response.get("sdk_error_code", 0)
+
+                if sdk_error_code != INVALID_OPTIMIZATION_ID:
+                    LOGGER.debug(
+                        "Error while getting the optimizer best %s", exc, exc_info=True
+                    )
 
                 count += 1
                 time.sleep(1)
