@@ -27,7 +27,7 @@ import random
 import re
 import tempfile
 import time
-from collections import defaultdict
+from collections import Mapping, defaultdict
 from datetime import datetime
 
 import six
@@ -40,6 +40,7 @@ from ._typing import (
     Callable,
     Dict,
     Generator,
+    Iterable,
     List,
     Optional,
     Sequence,
@@ -834,10 +835,14 @@ class ConfusionMatrix(object):
             raise ValueError("if you give y_true OR y_predicted you must give both")
 
         if winner_function is not None:
-            self.winner_function = winner_function  # type: ignore
+            self.winner_function = winner_function
+        else:
+            self.winner_function = self.default_winner_function
 
         if index_to_example_function is not None:
-            self.index_to_example_function = index_to_example_function  # type: ignore
+            self.index_to_example_function = index_to_example_function
+        else:
+            self.index_to_example_function = self.default_index_to_example_function
 
         self.y_true = y_true
         self.y_predicted = y_predicted
@@ -877,9 +882,9 @@ class ConfusionMatrix(object):
         Clear the caches.
         """
         # Set of indices (ints):
-        self._cache = set()
+        self._cache = set()  # type: Set[int]
         # Map index (int) -> example
-        self._cache_example = {}
+        self._cache_example = {}  # type: Dict[int, Any]
 
     def initialize(self):
         """
@@ -889,7 +894,7 @@ class ConfusionMatrix(object):
         self._matrix = self._create_matrix((n, n), 0)
         self._example_matrix = self._create_matrix((n, n), None)
 
-    def winner_function(self, ndarray):
+    def default_winner_function(self, ndarray):
         """
         A default winner function. Takes a list
         of patterns to apply winner function to.
@@ -899,13 +904,12 @@ class ConfusionMatrix(object):
 
         Returns a list of winning categories.
         """
-        try:
-            from numpy import argmax
+        if HAS_NUMPY:
 
             def winner(ndarray):
-                return argmax(ndarray, axis=1)
+                return numpy.argmax(ndarray, axis=1)
 
-        except ImportError:
+        else:
             # numpy is faster, but not required
             log_once_at_level(
                 logging.INFO,
@@ -920,7 +924,7 @@ class ConfusionMatrix(object):
 
         return winner(ndarray)
 
-    def index_to_example_function(self, index, **kwargs):
+    def default_index_to_example_function(self, index, **kwargs):
         """
         User-provided function.
 
@@ -1018,7 +1022,8 @@ class ConfusionMatrix(object):
                 self.type = "image"
         else:
             raise ValueError(
-                "index_to_example_function must return an int, string, URL, or {'sample': string, 'assetId': string}"
+                "index_to_example_function must return an int, string, URL, or {'sample': string, 'assetId': string}, not %r"
+                % type(example)
             )
         if self.type != "integer" and self.use_cache:
             self._put_example_in_cache(index, example)
@@ -1052,6 +1057,7 @@ class ConfusionMatrix(object):
         return example
 
     def _create_matrix(self, shape, initial_value):
+        # type: (Tuple[int, int], Any) -> List[List[Any]]
         """
         Create a matrix with initial value of initial_value.
 
@@ -1093,6 +1099,7 @@ class ConfusionMatrix(object):
         self._cache_example[key] = example
 
     def _example_from_list(self, indices, x, y, **kwargs):
+        # type: (Set[int], int, int, Any) -> Set[int]
         """
         Example from indices so that it is no more than max length.
         Use previous indices from cache.
@@ -1165,11 +1172,23 @@ class ConfusionMatrix(object):
 
         # Keep track of all indices for each cell:
         # Create confusion matrix:
-        xs = self.winner_function(y_true)
-        ys = self.winner_function(y_predicted)
+        xs = self.winner_function(y_true)  # type: Iterable[Any]
+        ys = self.winner_function(y_predicted)  # type: Iterable[Any]
+
+        examples = defaultdict(set)  # type: Dict[Tuple[int, int], Set[int]]
+        for (i, (raw_x, raw_y)) in enumerate(zip(xs, ys)):
+            try:
+                x = convert_to_scalar(raw_x, dtype=int)  # type: int
+            except TypeError:
+                LOGGER.warning("Invalid y_true value %r, ignoring it", raw_x)
+                continue
+
+            try:
+                y = convert_to_scalar(raw_y, dtype=int)  # type: int
+            except TypeError:
+                LOGGER.warning("Invalid y_predictor value %r, ignoring it", raw_y)
+                continue
 
-        examples = defaultdict(set)
-        for (i, (x, y)) in enumerate(zip(xs, ys)):
             # Add count to cell:
             self._matrix[x][y] += 1
             # Add index to cell:
@@ -1300,16 +1319,28 @@ class Histogram(object):
         Initialize the values of bins and data structures.
 
         Args:
-            start: float (optional), value of start range. Default 1e-12
-            stop: float (optional), value of stop range. Default 1e20
-            step: float (optional), value of step. Creates an
+            start: float (optional, deprecated), value of start range. Default 1e-12
+            stop: float (optional, deprecated), value of stop range. Default 1e20
+            step: float (optional, deprecated), value of step. Creates an
                 exponentially-distributed set of bins. Must be greater
                 than 1.0. Default 1.1
             offset: float (optional), center of distribution. Default is zero.
         """
+        if start != 1e-12 or stop != 1e20 or step != 1.1:
+            log_once_at_level(
+                logging.WARNING,
+                "Histogram will deprecate changing start, stop, or step values in a future version",
+            )
+
         if step <= 1.0:
             raise ValueError("Histogram step must be greater than 1.0")
 
+        if start <= 0:
+            raise ValueError("Histogram start must be greater than 0.0")
+
+        if stop <= start:
+            raise ValueError("Histogram stop must be greater than start")
+
         self.start = start
         self.stop = stop
         self.step = step
@@ -1339,8 +1370,139 @@ class Histogram(object):
         Clear the counts, initializes back to zeros.
         """
         self.counts = [0] * len(self.values)
+        if HAS_NUMPY:
+            self.counts = numpy.array(self.counts)
 
-    def add(self, values, counts=None, max_skip_count=10):
+    def get_bin_index(self, value):
+        """
+        Given a value, return the bin index where:
+
+            values[index] <= value < values[index + 1]
+        """
+        midpoint = len(self.counts) // 2
+        if value >= self.stop:
+            return len(self.values) - 2  # slight asymmetry
+        elif value <= -self.stop:
+            return 0
+        base = self.step
+        if value - self.offset == 0:
+            return midpoint
+        elif value - self.offset < 0:
+            return (
+                midpoint
+                - 1
+                - int(math.ceil(math.log(abs(value - self.offset) / self.start, base)))
+            )
+        else:
+            return (
+                int(math.ceil(math.log((value - self.offset) / self.start, base)))
+                + midpoint
+            )
+
+    def _add_via_python(self, values, counts=None, max_skip_count=None):
+        # type: (List, Optional[List], int) -> None
+        """
+        Values is a Python list. Only used
+        when numpy is not available.
+        """
+        max_skip_count = max_skip_count if max_skip_count is not None else 10
+
+        # Sort for speed of inserts
+        if counts is None:
+            values.sort()
+
+        # Find initial bin:
+        bucket = self.get_bin_index(values[0])
+
+        # Avoid attributes lookup at every loop
+        self_values = self.values
+        self_counts = self.counts
+
+        for i, value in enumerate(values):
+            skip_count = 0
+            while not (
+                (bucket == 0 and value <= self_values[1])
+                or (self_values[bucket] <= value < self_values[bucket + 1])
+            ):
+                skip_count += 1
+                if skip_count > max_skip_count:
+                    # if too many skips
+                    bucket = self.get_bin_index(value)
+                    break
+                else:
+                    bucket += 1
+
+            if counts is not None:
+                self_counts[bucket] += int(counts[i])
+            else:
+                self_counts[bucket] += 1
+
+    def _add_via_numpy(self, values, counts=None):
+        # type: (numpy.ndarray, Optional[numpy.ndarray]) -> None
+        """
+        Assumes numpy values (and counts).
+        """
+        midpoint = len(self.counts) // 2
+        base = numpy.log(self.step)
+
+        # Positive values (values > offset)
+
+        # The filter is an array of boolean same size as values
+        positive_values_filter = numpy.bitwise_and(
+            values > self.offset, values < self.stop
+        )
+
+        # This is a numpy array with only positive values and we center them
+        # around the offset (default 0 so no-op)
+        positive_values = values[positive_values_filter] - self.offset
+
+        # Then we transform all values to the suite index, and we add the
+        # midpoint as we are storing positives (values > offset) at the end of
+        # the self.values array
+        positive_indices = (
+            numpy.ceil(numpy.log(positive_values / self.start) / base).astype(int)
+            + midpoint
+        )
+
+        # Negative values (values > offset)
+
+        # The filter is an array of boolean same size as values
+        negative_values_filter = numpy.bitwise_and(
+            values < self.offset, values > -self.stop
+        )
+
+        # This is a numpy array with only negatives values and we center them
+        # around the offset (default 0 so no-op)
+        negative_values = values[negative_values_filter] - self.offset
+
+        # Then we transform all values to the suite index, and substract the
+        # index from the midpoint as we are storing positives (values > offset)
+        # at the end of the self.values array
+        negative_indices = (
+            midpoint
+            - 1
+            - numpy.ceil(
+                numpy.log(numpy.absolute(negative_values) / self.start) / base
+            ).astype(int)
+        )
+
+        # Values equals to the offset
+        zero_values = values[values == self.offset]
+
+        if counts is not None:
+            for i, match in enumerate(positive_values_filter):
+                if match:
+                    self.counts[positive_indices[i]] += counts[i]
+            for i, match in enumerate(negative_values_filter):
+                if match:
+                    self.counts[negative_indices[i]] += counts[i]
+            self.counts[midpoint] += counts[zero_values]
+        else:
+            numpy.add.at(self.counts, positive_indices, 1)
+            numpy.add.at(self.counts, negative_indices, 1)
+            self.counts[midpoint] += len(zero_values)
+
+    def add(self, values, counts=None, max_skip_count=None):
         """
         Add the value(s) to the count bins.
 
@@ -1350,14 +1512,12 @@ class Histogram(object):
             counts: a list of counts for each value in values. Triggers
                 special mode for conversion from Tensorboard
                 saved format. Assumes values are in order (min to max).
-            max_skip_count: int, (optional) maximum number of empty
-                cells that triggers a binary search.
+            max_skip_count: int, (optional) maximum number of missed
+                bins that triggers get_bin_index() (only used
+                when numpy is not available).
 
-        Counting values in bins can be expensive, so this method uses
-        binary_search to find the initial bin, and iterates through
-        sorted values, incrementing the bin as it goes. If too many
-        bins are empty (skip_count reaches max_skip_count) then it
-        jumps out and does another binary_search.
+         Counting values in bins can be expensive, so this method uses
+        numpy where possible.
         """
         try:
             values = [float(values)]
@@ -1367,37 +1527,21 @@ class Histogram(object):
         if len(values) == 0:
             return
 
+        # Returns numpy array, if possible:
         values = fast_flatten(values)
 
-        # Sort for speed of inserts
-        if counts is None:
-            values = sorted(values)
-        # Find initial bin:
-        bucket = self.get_bin_index(values[0])
-
-        for i, value in enumerate(values):
-            skip_count = 0
-            while not (self.values[bucket] <= value < self.values[bucket + 1]):
-                skip_count += 1
-                # if too many skips
-                if skip_count > max_skip_count:
-                    # then let's just do a binary search
-                    bucket = self.get_bin_index(value)
-                    break
-                else:
-                    bucket += 1
+        if HAS_NUMPY:
             if counts is not None:
-                self.counts[bucket] += int(counts[i])
-            else:
-                self.counts[bucket] += 1
+                counts = numpy.array(counts)
+            self._add_via_numpy(values, counts)
+        else:
+            self._add_via_python(values, counts, max_skip_count)
 
     def counts_compressed(self):
         """
         Convert list of counts to list of [(index, count), ...].
         """
-        return [
-            [i, self.counts[i]] for i in range(len(self.counts)) if self.counts[i] > 0
-        ]
+        return [[i, int(count)] for (i, count) in enumerate(self.counts) if count > 0]
 
     @classmethod
     def from_json(cls, data, verbose=True):
@@ -1500,37 +1644,12 @@ class Histogram(object):
         """
         values = [-float("inf"), self.offset, float("inf")]
         value = self.start
-        while value <= self.stop:
+        while self.offset + value <= self.stop:
             values.insert(1, self.offset - value)
             values.insert(-1, self.offset + value)
             value *= self.step
         return values
 
-    def get_bin_index(self, value):
-        """
-        Given a value, return the bin index where:
-
-            values[index] <= value < values[index + 1]
-
-        Implemented using binary search.
-        """
-        if value == float("inf"):
-            return len(self.values) - 1
-        return self.binary_search(value, 0, len(self.values) - 1)
-
-    def binary_search(self, value, low, high):
-        """
-        Find value between low and high, via binary search.
-        """
-        while True:
-            middle = (high + low) // 2
-            if (high - low) <= 1:
-                return low
-            elif value < self.values[middle]:
-                high = middle
-            else:
-                low = middle
-
     def get_count(self, min_value, max_value):
         """
         Get the count (can be partial of bin count) of a range.
@@ -1943,7 +2062,7 @@ def lazy_flatten(iterable):
             yield value
         else:
             if hasattr(value, "flatten"):
-                value = value.flatten()
+                value = value.flatten()  # type: ignore
             try:
                 new_iterator = iter(value)
             except TypeError:
@@ -1975,23 +2094,24 @@ def fast_flatten(items):
         try:
             # Vector, Matrix conversion:
             items = numpy.array(items, dtype=float)
-            return items.reshape(-1).tolist()
+            # Return numpy array:
+            return items.reshape(-1)
         except Exception:
             try:
                 # Uneven conversion, 2 deep:
                 items = numpy.array([numpy.array(item) for item in items], dtype=float)
-                return items.reshape(-1).tolist()
+                return items.reshape(-1)
             except Exception:
                 # Fall through
                 LOGGER.debug(
                     "numpy unable to convert items in fast_flatten", exc_info=True
                 )
-
+                return numpy.array(flatten(items))
     else:
         log_once_at_level(
             logging.INFO, "numpy not installed; using a slower flatten",
         )
-    return flatten(items)
+        return flatten(items)
 
 
 def convert_to_list(items, dtype=None):
@@ -2020,6 +2140,9 @@ def convert_to_string(user_data):
     """
     Given an object, return it as a string.
     """
+    if isinstance(user_data, Mapping):
+        user_data = json.dumps(user_data, sort_keys=True)
+
     if hasattr(user_data, "numpy"):
         user_data = user_data.numpy()
 
@@ -2029,6 +2152,29 @@ def convert_to_string(user_data):
     return str(user_data)
 
 
+def convert_to_string_truncated(user_data, size):
+    # type: (Any, int) -> str
+    value = convert_to_string(user_data)
+    if len(value) > size:
+        LOGGER.warning("truncated string; too long: '%s'...", value)
+        indicator = " [truncated]"
+        if size < len(indicator):
+            value = value[:size]
+        else:
+            value = value[: size - len(indicator)] + indicator
+    return value
+
+
+def convert_to_string_key(user_data):
+    # type: (Any) -> str
+    return convert_to_string_truncated(user_data, 100)
+
+
+def convert_to_string_value(user_data):
+    # type: (Any) -> str
+    return convert_to_string_truncated(user_data, 1000)
+
+
 def convert_to_scalar(user_data, dtype=None):
     # type: (Any, Optional[Type]) -> Any
     """
@@ -2049,7 +2195,7 @@ def convert_to_scalar(user_data, dtype=None):
 
     # Then try to convert numpy object to a Python scalar
     try:
-        if HAS_NUMPY and isinstance(user_data, numpy.number):
+        if hasattr(user_data, "item") and callable(user_data.item):
             user_data = user_data.item()
     except Exception:
         LOGGER.debug(
